/*
 * Copyright (C) 2009, Texas Instruments, Incorporated
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
#include <common.h>
#include <asm/io.h>
#include <asm/arch/hardware.h>
#include "gpio.h"

#define DATA_MASK(x)			(0x1 << (x))
#define DATA_SET(x)				(0x1 << (x))

static void gpio_cfg_pin(int bank, int gpio, int dir)
{
	u32 addr;
	
	addr = GPIO_OE;
	
	if (bank == 1) {
		addr += (u32)GPIO1_BASE;
	} else if (bank == 2) {
		addr += (u32)GPIO2_BASE;
	} else {
		addr += (u32)GPIO0_BASE;
	}
	
	if (dir) 
		writel((readl(addr) & ~DATA_MASK(gpio)), addr);
	else
		writel((readl(addr) | DATA_SET(gpio)), addr);
}

void gpio_direction_output(int bank, int gpio, int en)
{
	u32 addr = 0;

	gpio_cfg_pin(bank, gpio, GPIO_OUTPUT);
	
	if (bank == 1) {
		addr = (u32)GPIO1_BASE;
	} else if (bank == 2) {
		addr = (u32)GPIO2_BASE;
	} else {
		addr = (u32)GPIO0_BASE;
	}
	
	if (en) 
		writel(DATA_SET(gpio), addr + GPIO_SETDATAOUT);
	else
		writel(DATA_SET(gpio), addr + GPIO_CLEARDATAOUT);
}

void gpio_direction_input(int bank, int gpio)
{
	gpio_cfg_pin(bank, gpio, GPIO_INPUT);
}

void gpio_set_value(int bank, int gpio, int value)
{
	u32 addr = 0;

	if (bank == 1) {
		addr = (u32)GPIO1_BASE;
	} else if (bank == 2) {
		addr = (u32)GPIO2_BASE;
	} else {
		addr = (u32)GPIO0_BASE;
	}
		
	if (value) 
		writel(DATA_SET(gpio), addr + GPIO_SETDATAOUT);
	else
		writel(DATA_SET(gpio), addr + GPIO_CLEARDATAOUT);
}

unsigned int gpio_get_value(int bank, int gpio)
{
	u32 addr, value = 0;
	
	addr = GPIO_DATAIN;
	
	if (bank == 1) {
		addr += (u32)GPIO1_BASE;
	} else if (bank == 2) {
		addr += (u32)GPIO2_BASE;
	} else {
		addr += (u32)GPIO0_BASE;
	}
	
	value = __raw_readl(addr);
	
	return !!(value & DATA_MASK(gpio));
}
