/*
 * Copyright (C) 2009, Texas Instruments, Incorporated
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation version 2.
 *
 * This program is distributed "as is" WITHOUT ANY WARRANTY of any
 * kind, whether express or implied; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include "evm_ddr_edmaTest.h"

/* 
 *   DDR Test programming for DM81XX
 */
#define EMIF_TEST_EMIF0_ADD_0 0x80000000
#define EMIF_TEST_EMIF0_ADD_1 0x80008000
#define EMIF_TEST_EMIF0_ADD_2 0x80080000

#define EMIF_TEST_EMIF1_ADD_0 0xC0000000
#define EMIF_TEST_EMIF1_ADD_1 0xC0008000
#define EMIF_TEST_EMIF1_ADD_2 0xC0080000

#define EMIF_TEST_PATTERN_0 0x0DAD0DAD
#define EMIF_TEST_PATTERN_1 0xCAFECAFE
#define EMIF_TEST_PATTERN_2 0x12344321

#define EMIF_TEST_PATTERN_PASS 0xC0C0C0C0
#define EMIF_TEST_PATTERN_FAIL 0xDEADDEAD



/* 
 *   EDMA3 programming for DM81XX
 */
#define DM81XX_EDMA3_CH_ID      0

#define DM81XX_EDMA3_QUEUE_ID   0

#define DM81XX_EDMA3_NUM_BYTES  4

#define DM81XX_EDMA3_WAIT_TIME  50

/* EDMA3 Channel Controller Registers */

/* Global Registers */
#define DM81XX_EDMA3_CC_BASE        (0x49000000)
#define DM81XX_EDMA3_DCHMAP0        (DM81XX_EDMA3_CC_BASE + 0x0100)
#define DM81XX_EDMA3_DMAQNUM0       (DM81XX_EDMA3_CC_BASE + 0x0240)
#define DM81XX_EDMA3_QUEPRI         (DM81XX_EDMA3_CC_BASE + 0x0284)
#define DM81XX_EDMA3_EMCR           (DM81XX_EDMA3_CC_BASE + 0x0308)
#define DM81XX_EDMA3_EMCRH          (DM81XX_EDMA3_CC_BASE + 0x030C)
#define DM81XX_EDMA3_QEMCR          (DM81XX_EDMA3_CC_BASE + 0x0314)
#define DM81XX_EDMA3_CCERRCLR       (DM81XX_EDMA3_CC_BASE + 0x031C)
#define DM81XX_EDMA3_QWMTHRA        (DM81XX_EDMA3_CC_BASE + 0x0620)

/* Shadow region mapping registers */
#define DM81XX_EDMA3_DRAE0          (DM81XX_EDMA3_CC_BASE + 0x0340)
#define DM81XX_EDMA3_DRAE0H         (DM81XX_EDMA3_CC_BASE + 0x0344)
#define DM81XX_EDMA3_DRAE1          (DM81XX_EDMA3_CC_BASE + 0x0348)
#define DM81XX_EDMA3_DRAE1H         (DM81XX_EDMA3_CC_BASE + 0x034C)
#define DM81XX_EDMA3_DRAE2          (DM81XX_EDMA3_CC_BASE + 0x0350)
#define DM81XX_EDMA3_DRAE2H         (DM81XX_EDMA3_CC_BASE + 0x0354)
#define DM81XX_EDMA3_DRAE3          (DM81XX_EDMA3_CC_BASE + 0x0358)
#define DM81XX_EDMA3_DRAE3H         (DM81XX_EDMA3_CC_BASE + 0x035C)
#define DM81XX_EDMA3_DRAE4          (DM81XX_EDMA3_CC_BASE + 0x0360)
#define DM81XX_EDMA3_DRAE4H         (DM81XX_EDMA3_CC_BASE + 0x0364)
#define DM81XX_EDMA3_DRAE5          (DM81XX_EDMA3_CC_BASE + 0x0368)
#define DM81XX_EDMA3_DRAE5H         (DM81XX_EDMA3_CC_BASE + 0x036C)
#define DM81XX_EDMA3_DRAE6          (DM81XX_EDMA3_CC_BASE + 0x0370)
#define DM81XX_EDMA3_DRAE6H         (DM81XX_EDMA3_CC_BASE + 0x0374)
#define DM81XX_EDMA3_DRAE7          (DM81XX_EDMA3_CC_BASE + 0x0378)
#define DM81XX_EDMA3_DRAE7H         (DM81XX_EDMA3_CC_BASE + 0x037C)

/* EDMA Channel Registers for Shadow region 5 */
#define DM81XX_EDMA3_ESR            (DM81XX_EDMA3_CC_BASE + 0x2A10)
#define DM81XX_EDMA3_ESRH           (DM81XX_EDMA3_CC_BASE + 0x2A14)
#define DM81XX_EDMA3_IESR           (DM81XX_EDMA3_CC_BASE + 0x2A60)
#define DM81XX_EDMA3_IESRH          (DM81XX_EDMA3_CC_BASE + 0x2A64)
#define DM81XX_EDMA3_IPR            (DM81XX_EDMA3_CC_BASE + 0x2A68)
#define DM81XX_EDMA3_IPRH           (DM81XX_EDMA3_CC_BASE + 0x2A6C)
#define DM81XX_EDMA3_ICR            (DM81XX_EDMA3_CC_BASE + 0x2A70)
#define DM81XX_EDMA3_ICRH           (DM81XX_EDMA3_CC_BASE + 0x2A74)

/* EDMA3 PaRAM entry base register */
#define DM81XX_EDMA3_PARAM_BASE     (DM81XX_EDMA3_CC_BASE + 0x4000)

#define OPT                         (0x00)
#define SRC                         (0x04)
#define A_B_CNT                     (0x08)
#define DST                         (0x0C)
#define SRC_DST_BIDX                (0x10)
#define LINK_BCNTRLD                (0x14)
#define SRC_DST_CIDX                (0x18)
#define CCNT                        (0x1C)

#define __raw_readInt(a)   (*(volatile unsigned int *)(a))
#define __raw_writeInt(a, v) (*(volatile unsigned int *)(a) = (v))

#define CM_ALWON_L3_FAST_CLKSTCTRL  (PRCM_BASE + 0x1430) 
#define CM_ALWON_TPCC_CLKCTRL       (PRCM_BASE + 0x15F4) 
#define CM_ALWON_TPTC0_CLKCTRL      (PRCM_BASE + 0x15F8)
#define CM_ALWON_TPTC1_CLKCTRL      (PRCM_BASE + 0x15FC)
#define CM_ALWON_TPTC2_CLKCTRL      (PRCM_BASE + 0x1600)
#define CM_ALWON_TPTC3_CLKCTRL      (PRCM_BASE + 0x1604)

void DM81XX_PrcmEDMAEnable()
{

 __raw_writeInt(CM_ALWON_L3_FAST_CLKSTCTRL,   2);
 __raw_writeInt(CM_ALWON_TPCC_CLKCTRL,   2);
 __raw_writeInt(CM_ALWON_TPTC0_CLKCTRL,   2);
 __raw_writeInt(CM_ALWON_TPTC1_CLKCTRL,   2);
 __raw_writeInt(CM_ALWON_TPTC2_CLKCTRL,   2);
 __raw_writeInt(CM_ALWON_TPTC3_CLKCTRL,   2);
   return;
}

/* EDMA3 set params */
                                                                   
static void DM81XX_EDMA3_setParams(int chId,
                            int dmaQueue,
                            unsigned int srcAddr,
                            unsigned int dstAddr,
                            unsigned short edmaWidth,
                            unsigned short edmaHeight,
                            short srcLineOffset, short dstLineOffset)
{
    volatile unsigned int PaRAMEntryAddr =
        DM81XX_EDMA3_PARAM_BASE + (chId * 0x20);
    volatile unsigned int dchmapChId = DM81XX_EDMA3_DCHMAP0 + (chId * 4);

    volatile unsigned int dmaQnum = DM81XX_EDMA3_DMAQNUM0 + ((chId / 8) * 4);

    /* PaRAM entry setup */
    *((volatile unsigned int *) (PaRAMEntryAddr + OPT)) = \
                               ((0) |    
                               /* SAM ->  INCR mode */

                               (0 << 1) |   
                               /* DAM INCR mode */

                               (1 << 2) |   
                               /* SYNCDIM AB synchronized */

                               (0 << 3) |   
                               /* Static/Non-Static -> non-static mode */

                               (4 << 8) |   
                               /* FWID FIFO width is 128 bits */
           
                               (0 << 11) |  
                               /* TCCMODE Normal completion */

                               (chId << 12) |   
                               /* TCC Transfer Completion Code */

                               (1 << 20) |  
                               /* TCINTEN Transfer Complete Interrupt is enabled */

                               (0 << 21) |  
                               /* ITCINTEN Intermediate transfer complete interrupt is disabled */

                               (0 << 22) |  
                               /* TCCHEN Transfer Complete Chaining is disabled */

                               (0 << 23) |  
                               /* ITCCHEN Intermediate transfer complete chaining is disabled */

                               (0 << 24));  
                               /* PRIVID Privilege identification for the external host/CPU/DMA that programmed this Param */

    *((volatile unsigned int *) (PaRAMEntryAddr + SRC)) = srcAddr;
    *((volatile unsigned int *) (PaRAMEntryAddr + A_B_CNT)) =
        (edmaHeight << 16) | (edmaWidth);
    *((volatile unsigned int *) (PaRAMEntryAddr + DST)) = dstAddr;
    *((volatile unsigned int *) (PaRAMEntryAddr + SRC_DST_BIDX)) =
        (dstLineOffset << 16) | (srcLineOffset);
    *((volatile unsigned int *) (PaRAMEntryAddr + LINK_BCNTRLD)) =
        (0 << 16) | 0xFFFF;
    *((volatile unsigned int *) (PaRAMEntryAddr + SRC_DST_CIDX)) =
        (0 << 16) | 0;
    *((volatile unsigned int *) (PaRAMEntryAddr + CCNT)) = 1;

    if (chId < 32)
    {
        /* Set Shadow region for the channel */
        *((volatile unsigned int *) DM81XX_EDMA3_DRAE5) |= (1 << chId);

        /* Enable interrupt */
        *((volatile unsigned int *) DM81XX_EDMA3_IESR) |= (1 << chId);
    }
    else
    {
        /* Set Shadow region for the channel */
        *((volatile unsigned int *) DM81XX_EDMA3_DRAE5H) |= (1 << (chId - 32));

        /* Enable interrupt */
        *((volatile unsigned int *) DM81XX_EDMA3_IESRH) |= (1 << (chId - 32));
    }

    /* channel to PaRAM entry mapping */
    *((volatile unsigned int *) dchmapChId) = (chId << 5); 
    /* maintainig one to one mapping between channel and PaRAM entry */

    /* channel to queue mapping */
    *((volatile unsigned int *) dmaQnum) |= (dmaQueue << ((chId % 8) * 4));
}

/* EDMA3 trigger transfer */
static void DM81XX_EDMA3_triggerTransfer(int chId, int wait, int * transferStatus)
{
    volatile unsigned int data;

    if (chId < 32)
    {
        /* clear the interrupt */
        *((volatile unsigned int *) DM81XX_EDMA3_ICR) |= (1 << chId);

        /* Set event */
        *((volatile unsigned int *) DM81XX_EDMA3_ESR) |= (1 << chId);

        /* Wait for the interrupt */
        do
        {
            data = *((volatile unsigned int *) DM81XX_EDMA3_IPR);
            wait--;
            if(wait == 0)
               break;
        } while ((data & (1 << chId)) == 0);

        /* clear the interrupt */
        *((volatile unsigned int *) DM81XX_EDMA3_ICR) |= (1 << chId);
    }
    else
    {
        /* clear the interrupt */
        *((volatile unsigned int *) DM81XX_EDMA3_ICRH) |= (1 << (chId - 32));

        /* Set event */
        *((volatile unsigned int *) DM81XX_EDMA3_ESRH) |= (1 << (chId - 32));

        /* Wait for the interrupt */
        do
        {
            data = *((volatile unsigned int *) DM81XX_EDMA3_IPRH);
            wait--;
            if(wait == 0)
               break;
        } while ((data & (1 << (chId - 32))) == 0);

        /* clear the interrupt */
        *((volatile unsigned int *) DM81XX_EDMA3_ICRH) |= (1 << (chId - 32));

    }

//  __raw_writel(wait, 0x80000070); /* Save Loop count taken to complete EDMA */
    if(wait)
       *transferStatus = 1;
    else
       *transferStatus = 0;
}
#if 0
/* Wait for EDMA3 transfer completion */
static void DM81XX_EDMA3_waitForTransfer(int chId, int wait)
{
    volatile unsigned int data;

    if (chId < 32)
    {
        /* Check status of transfer completion */
        do
        {
            data = *((volatile unsigned int *) DM81XX_EDMA3_IPR);

        } while ((data & (1 << chId)) == 0);

        /* clear the interrupt */
        *((volatile unsigned int *) DM81XX_EDMA3_ICR) |= (1 << chId);
    }
    else
    {
        /* Check status of transfer completion */
        do
        {
            data = *((volatile unsigned int *) DM81XX_EDMA3_IPRH);

        } while ((data & (1 << (chId - 32))) == 0);

        /* clear the interrupt */
        *((volatile unsigned int *) DM81XX_EDMA3_ICRH) |= (1 << (chId - 32));
    }
}
#endif
static void DM81XX_EDMA3_setTransfer(unsigned int srcAddr,
                              unsigned int dstAddr
                              )
{
   int chId;
   int dmaQueue;
   unsigned short edmaWidth;
   unsigned short edmaHeight;
   short srcLineOffset;
   short dstLineOffset;

   chId          = DM81XX_EDMA3_CH_ID;
   dmaQueue      = DM81XX_EDMA3_QUEUE_ID;
   edmaWidth     = DM81XX_EDMA3_NUM_BYTES; /* Transfering only 4 Bytes to test DDR */
   edmaHeight    = 1;                      /* 1-D transfer */
   srcLineOffset = 1;                      /* SRC_BIDX gap between successive byte at SRC */
   dstLineOffset = 1;                      /* DST_BIDX gap between successive byte at DST */

   DM81XX_EDMA3_setParams(chId,      // chId
                 dmaQueue,           // dmaQueue
                 srcAddr,            // srcAddr
                 dstAddr,            // dstAddr
                 edmaWidth,          // edmaWidth        A-CNT 
                 edmaHeight,         // edmaHeight       B-CNT
                 srcLineOffset,      // srcLineOffset    SRC_BIDX
                 dstLineOffset);     // dstLineOffset    DST_BIDX
}

static void test_ti81xx_sdram_ddr_edmaTest(int * emif0_test_pass, int * emif1_test_pass)
{
  int emif0_pass;
  int emif1_pass;

  int src_ocmc_test_0;
  int dest_ocmc_test_0;
  int transferStatus;

  /* Flag to keep status of tranfer completion. 
   *    0- Transfer Failed  1-Transfer successfull 
   */
  transferStatus = 0;

  /* EMIF-0 test suite */
  {

      /* EMIF-0 test pattren 1 */
      src_ocmc_test_0 = EMIF_TEST_PATTERN_0;

      /* copy from OCMC to DDR plane */
      DM81XX_EDMA3_setTransfer((int)&src_ocmc_test_0,  // srcAddr
                    (int)EMIF_TEST_EMIF0_ADD_0                   // dstAddr
                    );                                // NumBytes 1D transfer

      /* Trigger the edma transfer */
      DM81XX_EDMA3_triggerTransfer(0, DM81XX_EDMA3_WAIT_TIME, &transferStatus);

      /* copy from DDR to OCMC plane */
      DM81XX_EDMA3_setTransfer((int)EMIF_TEST_EMIF0_ADD_0,         // srcAddr
                    (int)&dest_ocmc_test_0           // dstAddr
                    );                                // NumBytes 1D transfer

      /* Trigger the edma transfer */
      DM81XX_EDMA3_triggerTransfer(0, DM81XX_EDMA3_WAIT_TIME, &transferStatus);

      emif0_pass = (src_ocmc_test_0 == dest_ocmc_test_0) && transferStatus; 

     __raw_writel(emif0_pass, 0x80000010); /* Store the result for debug  */

      /* EMIF-0 test pattren 2 */
      src_ocmc_test_0 = EMIF_TEST_PATTERN_1;

      /* copy from OCMC to DDR plane */
      DM81XX_EDMA3_setTransfer((int)&src_ocmc_test_0,  // srcAddr
                    (int)EMIF_TEST_EMIF0_ADD_1                   // dstAddr
                    );                                // NumBytes 1D transfer

      /* Trigger the edma transfer */
      DM81XX_EDMA3_triggerTransfer(0, DM81XX_EDMA3_WAIT_TIME, &transferStatus);

      /* copy from DDR to OCMC plane */
      DM81XX_EDMA3_setTransfer((int)EMIF_TEST_EMIF0_ADD_1,         // srcAddr
                    (int)&dest_ocmc_test_0           // dstAddr
                    );                                // NumBytes 1D transfer

      /* Trigger the edma transfer */
      DM81XX_EDMA3_triggerTransfer(0, DM81XX_EDMA3_WAIT_TIME, &transferStatus);

      emif0_pass &= (src_ocmc_test_0 == dest_ocmc_test_0) && transferStatus; 

     __raw_writel(emif0_pass, 0x80000014); /* Store the result for debug  */

     }

     /* EMIF-1 test suite */
    {

      /* EMIF-1 test pattren 1 */
      src_ocmc_test_0 = EMIF_TEST_PATTERN_0;

      /* copy from OCMC to DDR plane */
      DM81XX_EDMA3_setTransfer((int)&src_ocmc_test_0,  // srcAddr
                    (int)EMIF_TEST_EMIF1_ADD_0       // dstAddr
                    );                                // NumBytes 1D transfer

      /* Trigger the edma transfer */
      DM81XX_EDMA3_triggerTransfer(0, DM81XX_EDMA3_WAIT_TIME, &transferStatus);

      /* copy from DDR to OCMC plane */
      DM81XX_EDMA3_setTransfer((int)EMIF_TEST_EMIF1_ADD_0,         // srcAddr
                    (int)&dest_ocmc_test_0                      // dstAddr
                    );                                // NumBytes 1D transfer

      /* Trigger the edma transfer */
      DM81XX_EDMA3_triggerTransfer(0, DM81XX_EDMA3_WAIT_TIME, &transferStatus);

      emif1_pass = (src_ocmc_test_0 == dest_ocmc_test_0) && transferStatus; 

      __raw_writel(emif1_pass, 0x80000020); /* Store the result for debug  */

      /* EMIF-1 test pattren 2 */
      src_ocmc_test_0 = EMIF_TEST_PATTERN_1;

      /* copy from OCMC to DDR plane */
      DM81XX_EDMA3_setTransfer((int)&src_ocmc_test_0,  // srcAddr
                    (int)EMIF_TEST_EMIF1_ADD_1       // dstAddr
                    );                                // NumBytes 1D transfer

      /* Trigger the edma transfer */
      DM81XX_EDMA3_triggerTransfer(0, DM81XX_EDMA3_WAIT_TIME, &transferStatus);

      /* copy from DDR to OCMC plane */
      DM81XX_EDMA3_setTransfer((int)EMIF_TEST_EMIF1_ADD_1,         // srcAddr
                    (int)&dest_ocmc_test_0                       // dstAddr
                    );                                // NumBytes 1D transfer

      /* Trigger the edma transfer */
      DM81XX_EDMA3_triggerTransfer(0, DM81XX_EDMA3_WAIT_TIME, &transferStatus);

      emif1_pass &= (src_ocmc_test_0 == dest_ocmc_test_0) && transferStatus; 

      __raw_writel(emif1_pass, 0x80000024); /* Store the result for debug  */
     }

    if(emif0_pass)
       __raw_writel(EMIF_TEST_PATTERN_PASS, 0x80000018); /* Store the result for debug  */
    else
       __raw_writel(EMIF_TEST_PATTERN_FAIL, 0x80000018); /* Store the result for debug  */

    if(emif1_pass)
       __raw_writel(EMIF_TEST_PATTERN_PASS, 0x80000028); /* Store the result for debug  */
    else
       __raw_writel(EMIF_TEST_PATTERN_FAIL, 0x80000028); /* Store the result for debug  */


   /* While Loop to stop CPU at this point for Debug purpose */
//  __raw_writel(EMIF_TEST_PATTERN_0, 0x80000050);
//   while((__raw_readl(0x80000050)) == EMIF_TEST_PATTERN_0);

   *emif0_test_pass = emif0_pass;
   *emif1_test_pass = emif1_pass;

}
void test_ti81xx_sdram_ddr()
{
  int emif0_pass;
  int emif1_pass;

  test_ti81xx_sdram_ddr_edmaTest(&emif0_pass, &emif1_pass);

//  emif0_pass = 0; /* Activate this to simulate failure of EMIF-0 */
//  emif1_pass = 0; /* Activate this to simulate failure of EMIF-1 */


  if(emif0_pass && emif1_pass)
  {
    /* Both EMIF's are active */
    /* Program the DMM to for interleaved configuration */
    __raw_writel(0x0, DMM_LISA_MAP__0);
    __raw_writel(0x0, DMM_LISA_MAP__1);

     /* LISA MAP Register-2 settings */
     /*  |  0x80  |  0x64  |  0x03  |  0x00  | 
              |        ||       ||       | 
              |        ||       ||       +--Physical address base address, SDRAM controller address MSB
              |        ||       || 
              |        ||       |+--  Mapped on SDRC 0 and SDRC 1 (interleaved)
              |        ||       +---  Reserved
              |        |+-----------  128-byte interleaving
              |        +------------  1-GB section
              +---------------------  System Byte Base Address, System address MSB
      */     
    __raw_writel(0x80640300, DMM_LISA_MAP__2);

     /* LISA MAP Register-3 settings */
     /*  |  0xC0  |  0x64  |  0x03  |  0x20  | 
              |        ||       ||       | 
              |        ||       ||       +--Physical address base address, SDRAM controller address MSB
              |        ||       || 
              |        ||       |+--  Mapped on SDRC 0 and SDRC 1 (interleaved)
              |        ||       +---  Reserved
              |        |+-----------  128-byte interleaving
              |        +------------  1-GB section
              +---------------------  System Byte Base Address, System address MSB
      */     

    __raw_writel(0xC0640320, DMM_LISA_MAP__3);
  }
  else if(emif0_pass && !emif1_pass)
  {
    /* Program the DMM to for interleaved configuration */
    __raw_writel(0x0, DMM_LISA_MAP__0);
    __raw_writel(0x0, DMM_LISA_MAP__1);

     /* LISA MAP Register-2 settings */
     /*  |  0x80  |  0x50  |  0x01  |  0x00  | 
              |        ||       ||       | 
              |        ||       ||       +--Physical address base address, SDRAM controller address MSB
              |        ||       || 
              |        ||       |+--  Mapped on SDRC 0
              |        ||       +---  Reserved
              |        |+-----------  No interleaving
              |        +------------  512 MB section
              +---------------------  System Byte Base Address, System address MSB
      */     
    __raw_writel(0x80500100, DMM_LISA_MAP__2);

     /* LISA MAP Register-2 settings */
     /*  |  0xA0  |  0x50  |  0x01  |  0x00  | 
              |        ||       ||       | 
              |        ||       ||       +--Physical address base address, SDRAM controller address MSB
              |        ||       || 
              |        ||       |+--  Mapped on SDRC 0
              |        ||       +---  Reserved
              |        |+-----------  No interleaving
              |        +------------  512 MB section
              +---------------------  System Byte Base Address, System address MSB
      */     
    __raw_writel(0xA0500120, DMM_LISA_MAP__3); 
  }
  else if(!emif0_pass && emif1_pass)
  {
    /* Only EMIF 1 is active */
    /* Program the DMM to for interleaved configuration */
    __raw_writel(0x0, DMM_LISA_MAP__0);
    __raw_writel(0x0, DMM_LISA_MAP__1);

     /* LISA MAP Register-2 settings */
     /*  |  0x80  |  0x50  |  0x02  |  0x00  | 
              |        ||       ||       | 
              |        ||       ||       +--Physical address base address, SDRAM controller address MSB
              |        ||       || 
              |        ||       |+--  Mapped on SDRC 1
              |        ||       +---  Reserved
              |        |+-----------  No interleaving
              |        +------------  512 MB section
              +---------------------  System Byte Base Address, System address MSB
      */     
    __raw_writel(0x80500200, DMM_LISA_MAP__2);

     /* LISA MAP Register-2 settings */
     /*  |  0xA0  |  0x50  |  0x02  |  0x00  | 
              |        ||       ||       | 
              |        ||       ||       +--Physical address base address, SDRAM controller address MSB
              |        ||       || 
              |        ||       |+--  Mapped on SDRC 1
              |        ||       +---  Reserved
              |        |+-----------  No interleaving
              |        +------------  512 MB section
              +---------------------  System Byte Base Address, System address MSB
      */     
    __raw_writel(0xA0500220, DMM_LISA_MAP__3);
  }
// else
// {
   /* Both EMIF's are not active. Need to have mechanism to handle this. */
// }
   return;
}

