#include <logo_816x.h>

static void * handle = 0x0;


int ti816x_read_bmp_image(uint32_t bmp_source, uint32_t bmp_dest)
{
	unsigned long long bmp_size = 0;
	int cut_width = 0;
	int bmp_width = 0;
	int bmp_height = 0;
	int bmp_stride = 0;
	int bmp_data_offset = 0;
	int bmp_pixel_bits = 0;
	int pad	= 0;
	int image_type = 0;
	uint32_t image_start_addr;
	int row_no;
	int row_dest;
	int rows_cropped;

	if(bmp_read16(bmp_source) == BMP_SIGNATURE)
	{
		/*read size of bmp file*/
		bmp_size = (bmp_read64(bmp_source) >> 16) & 0xFFFFFFFF;
#ifdef DEBUG_BMP
		printf("\nfile size = %x",bmp_size);
#endif
		/*read data starting offset*/
		bmp_data_offset = bmp_read16(bmp_source + BMP_DATA_OFFSET);
#ifdef DEBUG_BMP
		printf("\ndata offset = %x",bmp_data_offset);
#endif

		/*read bmp width*/
		bmp_width  = bmp_read16(bmp_source + BMP_WIDTH_OFFSET);
#ifdef DEBUG_BMP
		printf("\ndword width  = %d",bmp_width);
#endif
		bmp_stride =  bmp_width * 3;

		/*read bmp height*/
		bmp_height = bmp_read16(bmp_source + BMP_HEIGHT_OFFSET );
		if (bmp_height < 0)
		{
			image_type = BMP_IMAGE_TOPDOWN;
			bmp_height *= -1;
		}
		else
		{
			image_type = BMP_IMAGE_BOTTOMUP;
			bmp_height *= 1;
		}

#ifdef DEBUG_BMP
		printf("\ndword height = %d",bmp_height);
#endif
		bmp_pixel_bits = bmp_read16(bmp_source + BMP_BITS_PER_PIXEL_OFFSET);

		/* check whether the number of bits used for pixel is 24 or not */
		if (bmp_pixel_bits != 24)
		{
			printf("\nERROR: Not able to read image, use 24-bit BMP Image");
			return -1;
		}
		pad = (bmp_size - (bmp_stride * bmp_height) - 54);
		pad /= bmp_height;

		/*
		   set background color
		 */
		memset((void *)(bmp_dest),LOGO_BGCOLOR, FRAME_SIZE);
		if(bmp_width > WIDTH)
		{
			cut_width = bmp_width - WIDTH;
			bmp_width = WIDTH;
			bmp_stride = WIDTH * 3;
		}
		if(bmp_height > HEIGHT)
		{
			row_no = bmp_height - 1;
			rows_cropped = bmp_height - HEIGHT;
			bmp_height = HEIGHT;
		}
		else
		{
			row_no  = bmp_height - 1;
			rows_cropped = 0;
		}

		/*image start offset*/
		image_start_addr = bmp_dest+ (STRIDE * ((HEIGHT - bmp_height) >> 1 )) + ((STRIDE - (bmp_stride)) >> 1);

		/* aligning the address with 24 bit sets */
		if ((image_start_addr - bmp_dest) % 3 == 1)
		{
			image_start_addr += 2;
		}
		else if ((image_start_addr - bmp_dest) % 3 == 2)
		{
			image_start_addr += 1;
		}

		/*copy image data row by row to destination offset*/
		if (image_type == BMP_IMAGE_BOTTOMUP)
		{
			for(row_no = (bmp_height-1), row_dest = 0; row_no >= 0; row_no--, row_dest++)
			{
				memcpy((uint32_t*) (image_start_addr + (row_dest * (STRIDE))),
				(uint32_t*) ((bmp_source + bmp_data_offset) + (row_no * (bmp_stride + (cut_width * 3)+pad))),
					(bmp_stride));
			}
		}
		else
		{
			for(row_no = 0, row_dest = 0; row_no <= (bmp_height-1); row_no++, row_dest++)
			{
				memcpy((uint32_t*) (image_start_addr + (row_dest * (STRIDE))),
				(uint32_t*) ((bmp_source + bmp_data_offset) + (row_no * (bmp_stride + (cut_width * 3)+pad))),
					(bmp_stride));
			}
		}
		return 0;
	}
	return -1;
}

void HdVpssClkEnable()
{
	unsigned int delay;
#ifdef DEBUG_VPSS
	printf("\t\nPRCM for HDVPSS is in Progress","Output",1,1,1);
#endif
	WR_MEM_32(CM_HDDSS_CLKSTCTRL,2); /*Enable Power Domain Transition for HDVPSS */

	WR_MEM_32(CM_ACTIVE_HDDSS_CLKCTRL,0); /*Enable HDVPSS Clocks*/
	delay = 1000;
	while(delay-- > 0);
	WR_MEM_32(CM_ACTIVE_HDDSS_CLKCTRL,2); /*Enable HDVPSS Clocks*/

	/*Check clocks are active*/
	while(((RD_MEM_32(CM_HDDSS_CLKSTCTRL) & 0xFF00) >> 8) != 0xFF) ;

	/* Check to see module is functional */
	while(((RD_MEM_32(CM_ACTIVE_HDDSS_CLKCTRL) & 0x70000) >> 16) != 0) ;
#ifdef DEBUG_VPSS
	printf("\t\nPRCM for HDVPSS is Done Successfully","Output",1,1,1);
#endif
}

void HdmiClkEnable()
{
#ifdef DEBUG_VPSS
	printf("\t\nPRCM for HDMI is in Progress","Output",1,1,1);
#endif
	WR_MEM_32(CM_HDMI_CLKSTCTRL,2); /*Enable Power Domain Transition for HDMI */
	WR_MEM_32(CM_ACTIVE_HDMI_CLKCTRL,2); /*Enable HDMI Clocks*/

	/*Check clocks are active*/
	while(((RD_MEM_32(CM_HDMI_CLKSTCTRL) & 0x300) >> 8) != 0x3) ;

	/* Check to see module is functional */
	while(((RD_MEM_32(CM_ACTIVE_HDMI_CLKCTRL) & 0x70000) >> 16) != 0) ;
#ifdef DEBUG_VPSS
	printf("\t\nPRCM for HDMI is Done Successfully","Output",1,1,1);
#endif
}

void HdVpss_VencD7425_VencA7425()
{
#ifdef DEBUG_VPSS
	printf("\t\nNetra Video PLL: VencD->297 VencA->297 Please Wait.....","Output",1,1,1);
#endif
	/*SYS_CLKOUT selection*/
	//WR_MEM_32(CM_CLKOUT_CTRL, 0xA2);  /* /5 divider is selected*/
	WR_MEM_32(CM_CLKOUT_CTRL, 0x0);  /* /5 divider is selected*/
	/*set VENCD and VENCA clock to 75.25 to support 720P-60*/
	VideoPLL(0x6E, 0x2, 0xB, 0x0, 0x5, 0xA, 0x0, 0x10, 0xA, 0x0, 0x10, 0x0, 0x0);
#ifdef DEBUG_VPSS
	printf("\t\nNetra Video PLL Configuration is Done.....","Output",1,1,1);
#endif
}

void VideoPLL(unsigned int VIDEO_N, unsigned int VIDEO_P, unsigned int  VIDEO_INTFREQ1, unsigned int VIDEO_FRACFREQ1, unsigned int VIDEO_MDIV1, unsigned int  VIDEO_INTFREQ2, unsigned int VIDEO_FRACFREQ2, unsigned int VIDEO_MDIV2, unsigned int  VIDEO_INTFREQ3, unsigned int VIDEO_FRACFREQ3, unsigned int VIDEO_MDIV3, unsigned int B1, unsigned C2)
{
	unsigned int video_pll_ctrl=0;

	/*Put the Video PLL in Bypass Mode*/
	video_pll_ctrl = RD_MEM_32(VIDEOPLL_CTRL);
	video_pll_ctrl &=0xFFFFFFFB;
	video_pll_ctrl |=4;
	WR_MEM_32(VIDEOPLL_CTRL, video_pll_ctrl);

	/*Bring Video PLL out of Power Down Mode*/
	video_pll_ctrl = RD_MEM_32(VIDEOPLL_CTRL);
	video_pll_ctrl &=0xFFFFFFF7;
	video_pll_ctrl |=8;
	WR_MEM_32(VIDEOPLL_CTRL, video_pll_ctrl);

	/*Program the Video PLL Multiplier and Pre-dividr value*/
	video_pll_ctrl = RD_MEM_32(VIDEOPLL_CTRL);
	video_pll_ctrl &=0xFF;
	video_pll_ctrl |=(VIDEO_N<<16) | (VIDEO_P<<8);
	WR_MEM_32(VIDEOPLL_CTRL, video_pll_ctrl);

	/*Bring the Video PLL Individual output clocks out of Power Down Mode
	  1->Power Down
	  0->Normal Mode i.e., out of Power Down Mode
	 */
	WR_MEM_32(VIDEOPLL_PWD, 0x0);

	/*Program the Video PLL to generate SYSCLK11 (RF Modulator), SYSCLK17 (SD_VENC), and STC1 source clocks*/
	WR_MEM_32(VIDEOPLL_FREQ1, (1<<31)|(1<<28)|(VIDEO_INTFREQ1<<24)| VIDEO_FRACFREQ1);

	/*Program Video PLL SYSCLK11 Post divider register*/
	WR_MEM_32(VIDEOPLL_DIV1, (1<<8)|VIDEO_MDIV1);

	/*Program the PLL to generate SYSCLK13 (HD_VENC_D clock) and STC0/STC1 source clocks */
	WR_MEM_32(VIDEOPLL_FREQ2, (1<<31)|(1<<28)|(VIDEO_INTFREQ2<<24)|VIDEO_FRACFREQ2);
	WR_MEM_32(VIDEOPLL_DIV2,  (1<<8)|VIDEO_MDIV2);

	/*Program the PLL to generate SYSCLK13 and STC0/STC1 source clocks */
	WR_MEM_32(VIDEOPLL_FREQ3, (1<<31)|(1<<28)|(VIDEO_INTFREQ3<<24)|VIDEO_FRACFREQ3);
	WR_MEM_32(VIDEOPLL_DIV3,  (1<<8)|VIDEO_MDIV3);

	WR_MEM_32(CM_SYSCLK13_CLKSEL,  C2);
	WR_MEM_32(CM_SYSCLK15_CLKSEL,  B1);

	/*Wait for PLL to Lock*/
	while( (RD_MEM_32(VIDEOPLL_CTRL) & 0x80) !=0x80 );

	/*Put the Main PLL in Normal(PLL) Mode*/
	video_pll_ctrl = RD_MEM_32(VIDEOPLL_CTRL);
	video_pll_ctrl &=0xFFFFFFFB;
	WR_MEM_32(VIDEOPLL_CTRL, video_pll_ctrl);
}

void SetDvo2PinMux()
{
	/* enable clocks on all units */
	vps_write32(VPS_CLKC_ENABLE, 0x01031FFF);
	/*tied the HDCOMP clock to HDMI*/
	vps_write32(VPS_CLKC_VENC_CLK_SELECT, 0xC050f);
	/* enable all video encoders */
	vps_write32(VPS_CLKC_VENC_ENABLE, 0xF);

#if 0
	WR_MEM_32(0x48140814, 2);
	WR_MEM_32(0x48140818, 2);
	WR_MEM_32(0x4814081c, 2);
	WR_MEM_32(0x48140820, 2);
	WR_MEM_32(0x48140824, 2);
	WR_MEM_32(0x48140828, 1);
	WR_MEM_32(0x4814082c, 1);
	WR_MEM_32(0x48140830, 1);
	WR_MEM_32(0x48140834, 1);
	WR_MEM_32(0x48140838, 1);
	WR_MEM_32(0x4814083c, 2);
	WR_MEM_32(0x48140840, 2);
	WR_MEM_32(0x48140844, 2);
	WR_MEM_32(0x48140848, 2);
	WR_MEM_32(0x4814084c, 2);
	WR_MEM_32(0x48140850, 2);
	WR_MEM_32(0x48140854, 1);
	WR_MEM_32(0x48140858, 1);
	WR_MEM_32(0x4814085c, 1);
	WR_MEM_32(0x48140860, 1);
	WR_MEM_32(0x48140864, 1);
	WR_MEM_32(0x48140868, 1);
	WR_MEM_32(0x4814086c, 1);
	WR_MEM_32(0x48140870, 1);
	WR_MEM_32(0x48140874, 1);
	WR_MEM_32(0x48140878, 2);
	WR_MEM_32(0x4814087c, 1);
	WR_MEM_32(0x48140880, 1);
	WR_MEM_32(0x48140884, 1);
	WR_MEM_32(0x48140888, 1);
	WR_MEM_32(0x4814088c, 1);
	WR_MEM_32(0x48140890, 1);
	WR_MEM_32(0x48140894, 1);
	WR_MEM_32(0x48140898, 1);
	WR_MEM_32(0x4814089c, 1);
	WR_MEM_32(0x481408b4, 2);
	WR_MEM_32(0x481408b8, 2);
	WR_MEM_32(0x481408bc, 2);
	WR_MEM_32(0x481408c0, 2);
	WR_MEM_32(0x481408c4, 2);
	WR_MEM_32(0x481409d4, 0);
	WR_MEM_32(0x481409d8, 0);
	WR_MEM_32(0x481409dc, 0);
	WR_MEM_32(0x481409e0, 0);
	WR_MEM_32(0x481409e4, 0);
	WR_MEM_32(0x481409e8, 0);
	WR_MEM_32(0x481409ec, 0);
	WR_MEM_32(0x481409f0, 0);
#endif
	/*enable the DAC_CORE*/
	WR_MEM_32(0x48140660, 0x1);
	/*set the HSYNC/VSYNC for HDCOMP, use DVO1 signal, this
	is for PG2.0 or above silicon only*/	
	if ( get_cpu_rev() >= PG2_0) {
		/*these settings matches DVR reference,
		it may not work with DM816X EVM*/
		WR_MEM_32(0x4814086c, 0x2);
		WR_MEM_32(0x48140870, 0x2);
		WR_MEM_32(0x48140724, 0x2);
	}
#ifdef DEBUG_VPSS
	printf("\t\nPinmux Configuration is Done.....","Output",1,1,1);
#endif
}

void configureHdmi(struct hdmi_cfg_params *cfgPtr)
{
	int status;

	handle = 0x0;
	initParams.wp_base_addr = HDMI_WP_0_REGS;
	initParams.core_base_addr = HDMI_CORE_0_REGS;
	initParams.phy_base_addr = HDMI_PHY_0_REGS;

	if (ti816x_hdmi_lib_init(&initParams) != 0x0)
	{
		while(1);
	}

	handle = ti816x_hdmi_lib_open( 0,cfgPtr,&status,0x0);
	if (handle == 0x0)
	{
		while(1);
	}
}

void startHdmi(void)
{
	int status;

	status =  ti816x_hdmi_lib_start( handle, 0x0);

	if (status == 0x0)
	{
#ifdef DEBUG_VPSS
		printf("\nStarted HDMI \n");
#endif
		if (handle == 0x0)
		{
			while(1);
		}
	}
	else
	{
		printf("Could not start HDMI \n");
	}

}

void configVen(int venc_offset, int useEmbeddedSync, char cmd[], int act_pix, int hfp, int hbp, int hsw,
			   int act_lines, int vfp, int vbp, int vsw)
{

	int totalpixels = act_pix + hfp + hbp + hsw;
	int totallines = act_lines + vfp + vbp + vsw;
	int temp;

	/*for PG2.0 Silicon, discreted sync must be used for HDCOMP*/	
	if ((PG2_0 <= get_cpu_rev()) && (venc_offset == 0x8000))
		useEmbeddedSync = 0;

	if (useEmbeddedSync != 0x0)
		vps_write32(venc_offset + 0, 0x44022033);
		else
		vps_write32(venc_offset + 0, 0x44032033);

	if (strcmp(cmd, "colorbar") == 0)  {
		temp = vps_read32(venc_offset + 0);
		vps_write32(venc_offset + 0, temp | 0x8000);
	}
	
	vps_write32(venc_offset + 4, 0x1FD01E24); 	
	vps_write32(venc_offset + 8, 0x02DC020C);
	vps_write32(venc_offset + 0xC, 0x00DA004A);
	vps_write32(venc_offset + 0x10, 0x020C1E6C);
	vps_write32(venc_offset + 0x14, 0x02001F88);
	vps_write32(venc_offset + 0x18, 0x00200000);
	/*remove SOG for HDCOMP*/
	if ((venc_offset == 0x8000) && (!useEmbeddedSync)) {
		vps_write32(venc_offset + 0x1c, 0xb00);
		vps_write32(venc_offset + 0x20, 0xb00);
		vps_write32(venc_offset + 0x24, 0xb00);
	} else{
		vps_write32(venc_offset + 0x1c, 0x184C0C77);
		vps_write32(venc_offset + 0x20, 0x1C0C0C30);
		vps_write32(venc_offset + 0x24, 0x1C0C0C30);
	}

	/*setup timings*/
	vps_write32(venc_offset + 0x28, (132 << 24) | (totallines << 12) | (totalpixels));
	vps_write32(venc_offset + 0x2C, (hsw << 24) | 
		(((vfp + vbp + vsw) * 2 + (act_lines >> 1)) << 12) |
		(vfp + vbp + vsw - 1));

	vps_write32(venc_offset + 0x30, (hsw << 24) | 
		((act_pix + 8) << 12) |
		(hsw + hbp - 3));

	vps_write32(venc_offset + 0x34, (totallines - 1));
	/*remove SOG for HDCOMP*/
	if ((venc_offset == 0x8000) && (!useEmbeddedSync))
		vps_write32(venc_offset + 0x38, 0x0);
	else
		vps_write32(venc_offset + 0x38, 0x38338);

	if (useEmbeddedSync)
		temp = hsw + hbp + hfp;
	else
		temp = hsw + hbp;
	vps_write32(venc_offset + 0x3c, ((hsw << 24) |
		(act_pix << 12) |	temp));


	vps_write32(venc_offset + 0x40, ((vsw + vfp  + vbp) << 12));
	vps_write32(venc_offset + 0x44, (act_lines << 12));
	vps_write32(venc_offset + 0x48, ((vfp << 12) | (vsw << 24)));
	vps_write32(venc_offset + 0x4c, (vsw << 24) | (1 << 12) | 1);
	vps_write32(venc_offset + 0x50, ((vfp*2 + vbp + vsw + (act_lines >> 1) + 4) << 12) |
		(vfp*2 + vbp + vsw + (act_lines >> 1)));
	if (venc_offset == 0x8000) 
		temp = hsw + hbp - 5;
	else {
		if (useEmbeddedSync)
			temp = hsw + hbp + hfp - 8;
		else
			temp = hsw + hbp - 8;
	}
	vps_write32(venc_offset + 0x54, (hsw << 24) |
			(act_pix << 12) |
			temp);

	vps_write32(venc_offset + 0x58, ((vfp + vbp + vsw) << 12) | 1);
	vps_write32(venc_offset + 0x5C,
		(act_lines << 12) | 
		(2*(vfp + vbp + vsw) + (act_lines >> 1)) );

	vps_write32(venc_offset + 0x60, (vsw << 24) | (1 << 12));
	vps_write32(venc_offset + 0x64, (vsw << 24) | 
		(4 << 12) | 
		((vfp + vbp + vsw + act_lines -1) >> 1));
}

int ti816x_vps_init()
{
	/* enable comp units */
	vps_write32(VPS_COMP_STATUS, 0
			| (1<<0)   // HDMI enable
			| (1 << 8)  /*HDCOMP*/
			);

	/* set background color */
	vps_write32(VPS_COMP_BGCOLOR, LOGO_BGCOLOR);

	return 0;
}

/**
 * Load firmware for VPDMA.
 */
static int ti816x_vpdma_load_firmware(uint32_t* desc_buffer)
{
	uint32_t* firmware;
	volatile uint32_t status;

    volatile uint32_t vpdma_firmware_size;
    uint16_t *vpdma_firmware;

	int repeat;
	int result = 0;
	int ok;

	/* allocate physical contiguous memory for firmware buffer */
	firmware = (uint32_t*)desc_buffer;
	if (!firmware)
	{
#ifdef DEBUG_VPSS
		printf("unable to allocate memory for VPDMA firmware\n");
#endif
		return -1;
	}

    /* default firmware */
    vpdma_firmware      = vpdmaFirmware_1AD;
    vpdma_firmware_size = vpdmaFirmware_1AD_size;

    #ifdef CONFIG_TI816X
    if(get_cpu_rev() >= PG2_0)
    {
        /* if TI816x and >= PG2.0 then load different firmware */
        vpdma_firmware      = vpdmaFirmware_1B2;
        vpdma_firmware_size = vpdmaFirmware_1B2_size;
    }
    #endif

	/* copy firmware to buffer */
	memcpy(firmware,vpdma_firmware, vpdma_firmware_size);
	/* Reset VPDMA module */
	ti816x_vps_reset_module( VPS_MODULE_VPDMA, 1);
	udelay(10);
	ti816x_vps_reset_module( VPS_MODULE_VPDMA, 0);

	/* load firmware */
	vpdma_write32(VPDMA_LIST_ADDR, (uint32_t)firmware);

	/* wait till firmware is loaded */
        for (ok=0, repeat=0; repeat<100 && !ok; repeat++)
	{
		status = vpdma_read32(VPDMA_LIST_ATTR);
		if ((status & 0x80000) == 0x80000)
		{
			ok = 1;
		}
		else
		{
			udelay(10);
		}
	}

        if (!ok)
    	{
#ifdef DEBUG_VPSS
    		printf("failed to load VPDMA firmware\n");
#endif
    		result = -1;
    	}

    printf("VPDMA Firmware Version = 0x%08x\n", vpdma_read32(VPDMA_PID));

	return result;
}

void ti816x_vps_reset_module(vps_module_t module, int assert_reset)
{
	uint32_t val = vps_read32(VPS_CLKC_RESET);

	if (assert_reset)
		val |= (1<<module);
	else
		val &= ~(1<<module);

	vps_write32(VPS_CLKC_RESET, val);
}

uint32_t ti816x_dispmgr_create_grpx_conf_descriptor(int grpx_unit, uint32_t* desc_buffer, uint32_t* payload_buffer, uint32_t xres, uint32_t yres)
{
	uint32_t size = 0;
	uint32_t* desc;
	uint32_t* payload;

	/* set up frame configuration descriptor */
	desc = desc_buffer;
	desc[0] = 0x1;   /* frame configuration address */
	desc[1] = 16;    /* data length */
	desc[2] = (uint32_t)payload_buffer;  /* payload location */
	desc[3] = 0
		| (0xB<<27)     /* packet type */
		| (0<<26)       /* indirect command */
		| (1<<24)       /* block type */
		| ((grpx_unit+1)<<16)  /* destination GRPX0..GRPX2 */
		| (1<<0)        /* payload size in 128 bit units */
		;
	size += 16;
	payload = payload_buffer;
	payload[0] = 0;      /* reserved */
	payload[1] = 0
		| (xres<<16)  /* frame width */
		| (yres<<0)   /* frame height */
			;
	payload[2] = 0;      /* reserved */
	payload[3] = 0
		| (0<<30)    /* progressive format */
		| (1<<31)    /* yes soft reset of GRPX unit */
		;

	return size;
}

uint32_t ti816x_dispmgr_setup_layers(uint32_t* desc_buffer,
                                     int stride,
                                     int width,
                                     int height,
                                     uint32_t dispTime,
                                     uint32_t dispFps,
                                     uint32_t paddr)
{
	uint32_t size = 0;
	uint32_t numDesc = 0, i =0;
	uint32_t* desc;
	int chan;
	int x = 0;
	int y = 0;

	uint32_t layer_ctrl;
	desc = desc_buffer ;
	layer_ctrl = 0x0000E400;

	/*
	 * set up data descriptors for GRPX units
	 */
	/* add src_viewport origin to surface start address */

#ifdef CONFIG_GRPX0
	layer_ctrl |= 1<<(2);  /* enable GRPXx layer */
#elif defined CONFIG_GRPX1
	layer_ctrl |= 1<<(1);  /* enable GRPXx layer */
#else
	layer_ctrl |= 1<<(0);  /* enable GRPXx layer */
#endif

#ifdef CONFIG_GRPX0
	chan = 29;
#elif defined CONFIG_GRPX1
	chan = 30;
#else
	chan = 31;
#endif

    /* Program the same data descriptor
        BOOT_LOGO_DISPLAY_TIME_IN_SECS x BOOT_LOGO_DISPLAY_FPS
        times.

       Buffer size for descriptors is VPDMA_DESC_BUFFER_SIZE.
        If size needed for descriptors is more than this
        then limit number of descriptors to fit within
        VPDMA_DESC_BUFFER_SIZE
    */
    numDesc = dispTime * dispFps;

    if(numDesc*VPDMA_DATA_DESC_SIZE > VPDMA_DESC_BUFFER_SIZE)
    {
        numDesc = VPDMA_DESC_BUFFER_SIZE/VPDMA_DATA_DESC_SIZE;
    }

    for(i=0; i<numDesc; i++)
    {
        /* set up data transfer descriptor */
        desc[0] = 0
            | (DATA_TYPE<<26)    // RGB-888
            | (0<<25)            // no notification
            | (0<<24)            // field number
            | (0<<23)            // 2-dimensional data
            | (0<<20)            // +1 line skip (even)
            | (0<<16)            // +1 line skip (odd)
            | (stride)           // line stride
            ;
        desc[1] = 0
            | (width<<16)        // line length in pixels
            | (height)           // number of rows
            ;
        desc[2] = paddr;
        desc[3] = 0
                | (0xA<<27)    // packet type
            | (0<<26)      // 1D mode
            | (0<<25)      // inbound direction
            | (chan<<16)   // channel
            | (0<<9)       // priority
            | (chan<<0)    // next channel
            ;
        desc[4] = 0
            | (width<<16)  // region width
            | (height)     // region height
            ;
        desc[5] = 0
            | (x<<16)      // horizontal start
            | (y<<0)       // vertical start
            ;
        desc[6] = 0
            | (1<<7)       // first region
            | (1<<8)       // last region
            ;
        desc[7] = 0; // alpha blending enable
    	desc += 8;
	    size += 32;
    }
	/*enable GRPX on both HDMI and HDCOMP*/
	vps_write32(VPS_COMP_HDMI_VOUT1, layer_ctrl);
	vps_write32(VPS_COMP_HDCOMP, layer_ctrl);
	return size;
}

int ti816x_vpdma_send_list(uint32_t* desc_buffer, int desc_len, int list_no)
{
	/* convert descriptor size from byte units to 128-bit units */
	desc_len = (desc_len+15)>>4;

	/* send list */
	vpdma_write32(VPDMA_LIST_ADDR,(uint32_t)desc_buffer);
	vpdma_write32(VPDMA_LIST_ATTR, 0
			| (list_no<<24)  /* list number */
			| (0<<16)        /* list type: normal */
			| (desc_len<<0)  /* list size */
			);

	return 0;
}


int ti816x_dispmgr_wait_for_list_complete(int dispno)
{
        uint32_t status;
	int repeat;
	int ok;

        for (ok=0, repeat=0; repeat<100 && !ok; repeat++)
	{
		status = vpdma_read32(VPDMA_LIST_ATTR);
		if ((status & 0x80000) == 0x80000)
		{
			ok = 1;
		}
		else
		{
			udelay(10);
		}
	}
        if(repeat == 100)
	    return -1;
	return 0;
}

static int do_colorbar(cmd_tbl_t * cmdtp,int flag, int argc, char *argv[])
{
	struct hdmi_cfg_params *params = NULL;
	params = &param720p60;

	/* enable vps clock */
	HdVpssClkEnable();
	/* enable hdmi clock */
	HdmiClkEnable();
	HdVpss_VencD7425_VencA7425();
	SetDvo2PinMux();
    configVen(0x6000, 0, argv[0], 1280, 110, 220, 40, 720, 5, 20, 5);
    configVen(0x8000, 1, argv[0], 1280, 110, 220, 40, 720, 5, 20, 5);

	configureHdmi(params);
	startHdmi();
	/* set the comp for hdmi */
	if(ti816x_vps_init() == -1)
	{
#ifdef DEBUG_VPSS
		printf("\nERROR ti816x_vps_init failed");
#endif
		return -1;
	}
	return 0;
}

static int do_boot_logo(cmd_tbl_t * cmdtp,int flag, int argc, char *argv[])
{
	uint32_t  desc_buffer_base;
    uint32_t *firmware_addr;
    uint32_t *list_addr;
	uint32_t *payload_buffer;
	uint32_t size;
	uint32_t image_source = 0;
	uint32_t disp_time = 0;
	uint32_t disp_fps = 0;
	uint32_t image_dest = 0;
	uint32_t vpdma_desc_dest = 0;
	uint32_t paddr;
	struct hdmi_cfg_params *params = NULL;
	params = &param720p60;
	/* VPDMA list number to use; please also check ISR */
	int g_vpdma_list_no = VPDMA_LIST_NUMBER;

#ifdef CONFIG_GRPX0
	uint32_t grpx_unit = 0;
#elif defined CONFIG_GRPX1
	uint32_t grpx_unit = 1;
#else
	uint32_t grpx_unit = 2;
#endif
	/* check whether the arguments passed are correct  */
	if (argc < 2)
	{
			cmd_usage(cmdtp);
			return 1;
	}
	else if (strcmp(argv[1],"on") == 0)
	{
		if (argc != 7)
		{
			cmd_usage(cmdtp);
			return 1;
		}
		else
		{
			/* address in memory where bmp image is loaded */
			image_source = simple_strtoul(argv[2], NULL, 16);
			image_dest = simple_strtoul(argv[3], NULL, 16);
			vpdma_desc_dest = simple_strtoul(argv[4], NULL, 16);

			disp_time = simple_strtoul(argv[5], NULL, 10);
            if(disp_time <= 0)
            {
				printf("Bootup splash display time is out of range, using default value(60s)\n");
				disp_time = DEFAULT_BOOT_LOGO_DISPLAY_TIME_IN_SECS;
			}

 			disp_fps = simple_strtoul(argv[6], NULL, 10);
            if((disp_fps > 60) || (disp_fps < 1))
            {
				printf("Bootup splash display FPS entered is out of range, using default value(60fps)\n");
				disp_fps = DEFAULT_BOOT_LOGO_DISPLAY_FPS;
			}
		}
	}
	else if (strcmp(argv[1],"off") == 0)
	{
		vpdma_write32(VPDMA_LIST_ATTR, vpdma_read32(VPDMA_LIST_ATTR)
                | (g_vpdma_list_no<<24)  /* list number */
                | (1<<20)  /* stop bit for the list */
                );
		return 0;
	}
	else
	{
		cmd_usage(cmdtp);
		return 1;
	}
	/* memory address for loading vpdma firmware and descriptors, payload */
    firmware_addr    = (uint32_t*)(vpdma_desc_dest+VPDMA_DESC_FIRMWARE_OFFSET);
    payload_buffer   = (uint32_t*)(vpdma_desc_dest+VPDMA_DESC_PAYLOAD_OFFSET);
    list_addr        = (uint32_t*)(vpdma_desc_dest+VPDMA_DESC_LIST_OFFSET);    

	/* memory address for displaying the image formatted on the hdmi, the offset from the source location is 2 * WIDTH * HEIGHT */
	//image_dest = image_source + 3 * 3 * WIDTH * HEIGHT;

	if(image_source == image_dest)
    {
		printf("\nERROR: <logo_read_address> & <final_bmp address> cannot be same\n");
		return -1;
	}
	if((image_source == 0) || (image_dest == 0))
    {
		printf("\nERROR: <logo_read_address> or <final_bmp address> is NULL\n");
		return -1;
	}
	if(ti816x_read_bmp_image(image_source,image_dest) == -1)
	{
		printf("\nERROR: reading BMP image failed\n");
		return -1;
	}

	/* enable vps clock */
	HdVpssClkEnable();
	/* enable hdmi clock */
	HdmiClkEnable();
	HdVpss_VencD7425_VencA7425();
	SetDvo2PinMux();
	configVen(0x6000, 0, argv[0], 1280, 110, 220, 40, 720, 5, 20, 5);
	configVen(0x8000, 1, argv[0], 1280, 110, 220, 40, 720, 5, 20, 5);
	configureHdmi(params);
	startHdmi();

	/* set the comp for hdmi */
	if(ti816x_vps_init() == -1)
	{
#ifdef DEBUG_VPSS
		printf("\nERROR ti816x_vps_init failed");
#endif
		return -1;
	}

	if (ti816x_vpdma_load_firmware(firmware_addr) == -1)
	{
#ifdef DEBUG_VPSS
		printf("\nERROR vpdma_load_firmware failed");
#endif
		return -1;
	}

	paddr = image_dest;

	/* Allocate physical memory for MV data */
	size = ti816x_dispmgr_create_grpx_conf_descriptor(grpx_unit, list_addr, payload_buffer,WIDTH,HEIGHT);
	ti816x_vpdma_send_list(list_addr, size, g_vpdma_list_no);

       	/* wait for list complete interrupt */
	if(ti816x_dispmgr_wait_for_list_complete(1) == -1)
	{
#ifdef DEBUG_VPSS
		printf("\nERROR : wait for list complete FAILED!");
#endif
		return -1;
	}

#ifdef CONFIG_GRPX0
	vpdma_write32(VPDMA_GRPX0_DATA_CSTAT,0x00001C00);
#elif defined CONFIG_GRPX1
	vpdma_write32(VPDMA_GRPX1_DATA_CSTAT,0x00001C00);
#else
	vpdma_write32(VPDMA_GRPX2_DATA_CSTAT,0x00001C00);
#endif

	size = ti816x_dispmgr_setup_layers(list_addr,STRIDE,WIDTH,HEIGHT+60,disp_time,disp_fps,paddr);
	ti816x_vpdma_send_list( list_addr, size, g_vpdma_list_no);
	while (ti816x_dispmgr_wait_for_list_complete(1)==-1);
	return 0;
}

U_BOOT_CMD(colorbar, 3, 1, do_colorbar,"Displays a colorbar using HDVPSS","");
U_BOOT_CMD(logo, 7, 0, do_boot_logo,"Displays a logo using HDVPSS",
"on <logo_read_address> <final_BMP DSS read address> <VPDMA Descriptor address> <Display time in seconds> <Display fps (less than 60)> \n"
"logo off");
