#include <command.h>
#include <common.h>
#include <hdmi_cfg.h>
#include <ti81xx_vpdma_firmware.h>
#include <asm/arch/sys_proto.h>

//#define CONFIG_GRPX0
#define CONFIG_720p

/* We will program the BOOT_LOGO_DISPLAY_TIME_IN_SECS*BOOT_LOGO_DISPLAY_FPS
    VPDMA descriptors

   Roughly on a 60FPS display like 1080p60 or 720p60
   this will result in boot logo being display for
    BOOT_LOGO_DISPLAY_TIME_IN_SECS seconds

   BOOT_LOGO_DISPLAY_TIME_IN_SECS*BOOT_LOGO_DISPLAY_FPS*32

    MUST BE <= VPDMA_DESC_BUFFER_SIZE
*/
#define DEFAULT_BOOT_LOGO_DISPLAY_TIME_IN_SECS  (60)
#define DEFAULT_BOOT_LOGO_DISPLAY_FPS           (60)

#define WR_MEM_32(addr, data)    *(unsigned int*)(addr) =(unsigned int)(data)
#define RD_MEM_32(addr)          *(unsigned int*)(addr)

#define PRCM_BASE_ADDR           0x48180000
#define CTRL_MODULE_BASE_ADDR    0x48140000
#define CM_CLKOUT_CTRL           (PRCM_BASE_ADDR + 0x100)

#define TI816X_HDMI_8BIT_720_60_16_9_HD {\
	TRUE, hdmi_720P_60_mode,\
	TRUE, TI816x_HDMIWPCONFIG_8BIT_IF_SALVE,\
	TRUE, TI816X_HDMICORE_IPCFG_8BIT_IF_SRCD_SYNC,\
	TRUE, TI816X_HDMICOREDATAPATHCONFIG_BYPS_ALL_8BIT_OUTPUT_BT709,\
	TRUE, TI816X_HDMIINFOFRAME_CFG_RGB_OVERSCAN_BT709_43}

struct hdmi_cfg_params param720p60 = TI816X_HDMI_8BIT_720_60_16_9_HD;
static struct ti816x_hdmi_init_params initParams;

#define VIDEOPLL_CTRL            (CTRL_MODULE_BASE_ADDR + 0x470)
#define VIDEOPLL_PWD             (CTRL_MODULE_BASE_ADDR + 0x474)
#define VIDEOPLL_FREQ1           (CTRL_MODULE_BASE_ADDR + 0x478)
#define VIDEOPLL_DIV1            (CTRL_MODULE_BASE_ADDR + 0x47C)
#define VIDEOPLL_FREQ2           (CTRL_MODULE_BASE_ADDR + 0x480)
#define VIDEOPLL_DIV2            (CTRL_MODULE_BASE_ADDR + 0x484)
#define VIDEOPLL_FREQ3           (CTRL_MODULE_BASE_ADDR + 0x488)
#define VIDEOPLL_DIV3            (CTRL_MODULE_BASE_ADDR + 0x48C)

/*HDVPSS*/
#define CM_HDDSS_CLKSTCTRL       (PRCM_BASE_ADDR + 0x0404)
#define CM_HDMI_CLKSTCTRL        (PRCM_BASE_ADDR + 0x0408)
#define CM_ACTIVE_HDDSS_CLKCTRL  (PRCM_BASE_ADDR + 0x0424)
#define CM_SYSCLK13_CLKSEL       (PRCM_BASE_ADDR + 0x0334)
#define CM_SYSCLK15_CLKSEL       (PRCM_BASE_ADDR + 0x0338)
#define CM_ACTIVE_HDMI_CLKCTRL   (PRCM_BASE_ADDR + 0x0428)

#define HDMI_REG_BASE            0x46C00000
#define HDMI_REG_SIZE            (4*1024)

#define hdmi_read32(off)         *(int*)(HDMI_REG_BASE+(off))
#define hdmi_write32(off, value) *( int*)(HDMI_REG_BASE+(off)) = (value)

#define vps_read32(off)          *(volatile int*)(VPS_REG_BASE+(off))
#define vps_write32(off, value)  *(volatile int*)(VPS_REG_BASE+(off)) = (value)

#define vpdma_read32(off)           *(volatile int*)(VPDMA_REG_BASE+(off))
#define vpdma_write32(off, value)   (*(volatile int*)(VPDMA_REG_BASE+(off)) =(value))

#define VPDMA_REG_BASE           0x4810D000
#define VPS_REG_BASE             0x48100000

#define VPDMA_LIST_ATTR          0x008
#define VPDMA_LIST_ADDR          0x004
#define VPDMA_LIST_NUMBER        2
#define VPS_COMP_HDMI_VOUT1      0x5204
#define VPS_COMP_HDCOMP          0x5208

#define VPDMA_DESC_FIRMWARE_OFFSET  (0)
#define VPDMA_DESC_LIST_OFFSET      (0)
#define VPDMA_DESC_PAYLOAD_OFFSET   (0x1000)
#define VPDMA_DESC_BUFFER_SIZE      (128*1024)

#define VPDMA_DATA_DESC_SIZE        (32) /* in bytes */

#define VPS_REG_BASE		0x48100000
#define VPS_REG_SIZE		(128*1024)

#define vps_read32(off)         *(volatile int*)(VPS_REG_BASE+(off))
#define vps_write32(off, value) *(volatile int*)(VPS_REG_BASE+(off)) = (value)

#define VPS_CLKC_ENABLE           0x0100
#define VPS_CLKC_VENC_CLK_SELECT  0x0114
#define VPS_CLKC_VENC_ENABLE      0x0118
#define VPS_COMP_STATUS           0x5200
#define VPS_COMP_BGCOLOR          0x5214	/*Register offset ! Please dont change it to the color code*/
#define LOGO_BGCOLOR		  0x0   	/*Black*/
#define VPS_CLKC_RESET            0x0104

#define vpdma_read32(off)         *(volatile int*)(VPDMA_REG_BASE+(off))
#define vpdma_write32(off, value) (*(volatile int*)(VPDMA_REG_BASE+(off)) =(value))
#define VPDMA_PID                 0x000
#define VPDMA_LIST_ADDR           0x004
#define VPDMA_LIST_ATTR           0x008
#define VPDMA_LIST_STAT_SYNC      0x00C
#define VPDMA_BG_RGB              0x018
#define VPDMA_GRPX0_DATA_CSTAT    0x37c
#define VPDMA_GRPX1_DATA_CSTAT    0x380
#define VPDMA_GRPX2_DATA_CSTAT    0x384

#define PRCM_REG_BASE             0x48180000
#define PRCM_REG_SIZE             (12*1024)

#define prcm_read32(off)          *(volatile int*)(PRCM_REG_BASE+(off))
#define prcm_write32(off, value)  (*(volatile int*)(PRCM_REG_BASE+(off)) =(value))

#define CM_HDVPSS_CLKSTCTRL       0x0800
#define CM_HDVPSS_HDVPSS_CLK_CTRL 0x0820
#define CM_HDVPSS_HDMI_CLKCTRL    0x0824

#define PM_HDVPSS_PWRSTCTRL       0x0E00
#define PM_HDVPSS_PWRSTST         0x0E04
#define RM_HDVPSS_RSTCTRL         0x0E10
#define RM_HDVPSS_RSTST           0x0E14

#define bmp_read64(off)		  *(long long*)(off)
#define bmp_read16(off)		  *(short int*)(off)

#define DATA_TYPE                 0x6

#ifdef CONFIG_480p
#define WIDTH                     720
#define HEIGHT                    480
#elif defined CONFIG_720p
#define WIDTH                     1280
#define HEIGHT                    720
#else
#define WIDTH                     1920
#define HEIGHT                    1080
#endif

#define STRIDE                    3*WIDTH
#define FRAME_SIZE                (STRIDE * HEIGHT)

#define bmp_read64(off)           *(long long*)(off)
#define bmp_read16(off)           *(short int*)(off)

#define BMP_SIGNATURE             0x4d42
#define BMP_WIDTH_OFFSET          0x12
#define BMP_HEIGHT_OFFSET         0x16
#define BMP_DATA_OFFSET           0xA
#define BMP_BITS_PER_PIXEL_OFFSET 0x1C
#define BMP_IMAGE_BOTTOMUP        0
#define BMP_IMAGE_TOPDOWN         1

typedef struct
 {
         uint32_t n;
         uint32_t m;
         uint32_t m2;
         uint32_t clk_ctrl;
 } pll_config_t;

typedef enum
{
       VPS_MODULE_VPDMA = 0
} vps_module_t;

static int ti816x_vpdma_load_firmware(uint32_t*);
int ti816x_vps_init(void);
int ti816x_set_mode(int ,int , int);
int ti816x_vpdma_send_list(uint32_t*, int, int);
int ti816x_dispmgr_wait_for_list_complete(int);
int ti816x_read_bmp_image(uint32_t, uint32_t);
uint32_t ti816x_dispmgr_create_grpx_conf_descriptor(int, uint32_t*, uint32_t*, uint32_t, uint32_t);
uint32_t ti816x_dispmgr_setup_layers(uint32_t*, int, int, int, uint32_t, uint32_t, uint32_t);
void HdVpssClkEnable(void);
void HdmiClkEnable(void);
void HdVpss_VencD297_VencA297(void);
void VideoPLL(unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned int, unsigned);
void SetDvo2PinMux(void);
void configureHdmi(struct hdmi_cfg_params*);
void startHdmi(void);
void configureHdVenc720P60(int, char [], int);
void ti816x_vps_reset_module(vps_module_t, int);
